@@ -62,6 +62,11 @@ public abstract class AbstractQuery<T> {
      */
     protected AbstractCriteria criteria;
 
+    /**
+     * Enable or disable trace.
+     */
+    protected boolean trace;
+
     public AbstractQuery(List<String> groups, String name, TimestampRange timestampRange, Set<String> tagProjections) {
         this.groups = groups;
         this.name = name;
@@ -100,6 +105,14 @@ public abstract class AbstractQuery<T> {
         return this;
     }
 
+    /**
+     * Enable trace for the query.
+     */
+    public AbstractQuery<T> enableTrace() {
+        this.trace = true;
+        return this;
+    }
+
     /**
      * @return QueryRequest for gRPC level query.
      * @throws BanyanDBException thrown from entity build, e.g. invalid reference to non-exist fields or tags.
@@ -152,7 +165,6 @@ public abstract class AbstractQuery<T> {
         return b.build();
     }
 
-    @RequiredArgsConstructor
     public static class OrderBy {
         /**
          * The field name for ordering.
@@ -163,9 +175,27 @@ public abstract class AbstractQuery<T> {
          */
         private final Sort type;
 
+        /**
+         * Create an orderBy condition with given rule name and sort type
+         */
+        public OrderBy(final String indexRuleName, final Sort type) {
+            this.indexRuleName = indexRuleName;
+            this.type = type;
+        }
+
+        /**
+         * Create an orderBy condition with timestamp and sort type
+         */
+        public OrderBy(final Sort type) {
+            this.indexRuleName = null;
+            this.type = type;
+        }
+
         BanyandbModel.QueryOrder build() {
             final BanyandbModel.QueryOrder.Builder builder = BanyandbModel.QueryOrder.newBuilder();
-            builder.setIndexRuleName(indexRuleName);
+            if (indexRuleName != null) {
+                builder.setIndexRuleName(indexRuleName);
+            }
             builder.setSort(
                     Sort.DESC.equals(type) ? BanyandbModel.Sort.SORT_DESC : BanyandbModel.Sort.SORT_ASC);
             return builder.build();
