@@ -18,7 +18,9 @@
 
 package org.apache.skywalking.banyandb.v1.client;
 
-import com.google.protobuf.Timestamp;
+import java.util.Map;
+import java.util.Optional;
+
 import lombok.Getter;
 import org.apache.skywalking.banyandb.common.v1.BanyandbCommon;
 import org.apache.skywalking.banyandb.model.v1.BanyandbModel;
@@ -27,38 +29,36 @@ import org.apache.skywalking.banyandb.v1.client.grpc.exception.InvalidReferenceE
 import org.apache.skywalking.banyandb.v1.client.metadata.MetadataCache;
 import org.apache.skywalking.banyandb.v1.client.metadata.Serializable;
 
-import java.util.Optional;
-
 public abstract class AbstractWrite<P extends com.google.protobuf.GeneratedMessageV3> {
     /**
-     * Group name of the current entity
+     * Timestamp represents the time of the current data point, in milliseconds.
+     * <p>
+     * <b>When to set:</b>
+     * <ul>
+     *   <li><b>Stream and Measure writes:</b> This field <i>must</i> be set to indicate the event time.</li>
+     *   <li><b>Trace writes:</b> This field is <i>not needed</i> and should be left unset; trace data does not require an explicit timestamp here.</li>
+     * </ul>
      */
     @Getter
-    protected final String group;
-    /**
-     * Owner name of the current entity
-     */
-    @Getter
-    protected final String name;
-    /**
-     * Timestamp represents the time of current stream
-     * in the timeunit of milliseconds.
-     */
-    @Getter
-    protected final long timestamp;
+    protected Optional<Long> timestamp;
 
     protected final Object[] tags;
 
     protected final MetadataCache.EntityMetadata entityMetadata;
 
-    public AbstractWrite(String group, String name, long timestamp) {
-        this.group = group;
-        this.name = name;
-        this.timestamp = timestamp;
-        this.entityMetadata = MetadataCache.INSTANCE.findMetadata(group, name);
-        if (this.entityMetadata == null) {
+    public AbstractWrite(MetadataCache.EntityMetadata entityMetadata, long timestamp) {
+        this(entityMetadata);
+        this.timestamp = Optional.of(timestamp);
+    }
+
+    /**
+     * Build a write request without initial timestamp.
+     */
+    AbstractWrite(MetadataCache.EntityMetadata entityMetadata) {
+        if (entityMetadata == null) {
             throw new IllegalArgumentException("metadata not found");
         }
+        this.entityMetadata = entityMetadata;
         this.tags = new Object[this.entityMetadata.getTotalTags()];
     }
 
@@ -73,12 +73,26 @@ public abstract class AbstractWrite<P extends com.google.protobuf.GeneratedMessa
 
     P build() {
         BanyandbCommon.Metadata metadata = BanyandbCommon.Metadata.newBuilder()
-                .setGroup(this.group).setName(this.name).build();
-        Timestamp ts = Timestamp.newBuilder()
-                .setSeconds(timestamp / 1000)
-                .setNanos((int) (timestamp % 1000 * 1_000_000)).build();
-        return build(metadata, ts);
+                .setGroup(entityMetadata.getGroup()).setName(entityMetadata.getName()).setModRevision(entityMetadata.getModRevision()).build();
+
+        return build(metadata);
     }
 
-    protected abstract P build(BanyandbCommon.Metadata metadata, Timestamp ts);
+    protected abstract P build(BanyandbCommon.Metadata metadata);
+
+    @Override
+    public String toString() {
+        StringBuilder stringBuilder = new StringBuilder();
+        stringBuilder.append("group=").append(entityMetadata.getGroup()).append(", ").append("name=")
+                .append(entityMetadata.getName()).append(", ").append("timestamp=").append(timestamp).append(", ");
+        for (int i = 0; i < this.tags.length; i++) {
+            final int index = i;
+            Map<String, MetadataCache.TagInfo> tagMap = this.entityMetadata.getTagOffset();
+            Optional<String> tagName = tagMap.keySet().stream().filter(name -> tagMap.get(name).getOffset() == index).findAny();
+            if (tagName.isPresent()) {
+                stringBuilder.append(tagName.get()).append("=").append(((Serializable<BanyandbModel.TagValue>) this.tags[i]).serialize()).append(", ");
+            }
+        }
+        return stringBuilder.toString();
+    }
 }
