@@ -50,6 +50,8 @@ public class MeasureQuery extends AbstractQuery<BanyandbMeasure.QueryRequest> {
 
     private OrderBy orderBy;
 
+    private Set<String> stages;
+
     public MeasureQuery(final List<String> groups, final String name, final Set<String> tagProjections, final Set<String> fieldProjections) {
         this(groups, name, null, tagProjections, fieldProjections);
     }
@@ -110,7 +112,7 @@ public class MeasureQuery extends AbstractQuery<BanyandbMeasure.QueryRequest> {
         Preconditions.checkArgument(fieldProjections.contains(field), "field should be selected first");
         Preconditions.checkArgument(this.tagProjections.containsAll(groupByKeys), "groupBy tags should be selected first");
         Preconditions.checkState(this.aggregation == null, "aggregation should only be set once");
-        this.aggregation = new Aggregation(field, Aggregation.Type.COUNT, groupByKeys);
+        this.aggregation = new Aggregation(field, Aggregation.Type.SUM, groupByKeys);
         return this;
     }
 
@@ -136,8 +138,8 @@ public class MeasureQuery extends AbstractQuery<BanyandbMeasure.QueryRequest> {
         return this;
     }
 
-    public MeasureQuery orderBy(String tagName, Sort sort) {
-        this.orderBy = new OrderBy(tagName, sort);
+    public MeasureQuery stages(Set<String> stages) {
+        this.stages = stages;
         return this;
     }
 
@@ -166,6 +168,9 @@ public class MeasureQuery extends AbstractQuery<BanyandbMeasure.QueryRequest> {
                     .addAllNames(fieldProjections)
                     .build());
         }
+        if (stages != null && !stages.isEmpty()) {
+            builder.addAllStages(stages);
+        }
         if (this.aggregation != null) {
             BanyandbMeasure.QueryRequest.GroupBy.Builder groupByBuilder = BanyandbMeasure.QueryRequest.GroupBy.newBuilder()
                     .setTagProjection(buildTagProjections(entityMetadata, this.aggregation.groupByTagsProjection));
@@ -200,6 +205,7 @@ public class MeasureQuery extends AbstractQuery<BanyandbMeasure.QueryRequest> {
         }
         // add all criteria
         buildCriteria().ifPresent(builder::setCriteria);
+        builder.setTrace(this.trace);
         return builder.build();
     }
 
