@@ -19,17 +19,18 @@
 package org.apache.skywalking.banyandb.v1.client;
 
 import com.google.protobuf.Timestamp;
+import java.util.Deque;
+import java.util.LinkedList;
+import java.util.Optional;
 
 import lombok.Getter;
 import org.apache.skywalking.banyandb.common.v1.BanyandbCommon;
 import org.apache.skywalking.banyandb.model.v1.BanyandbModel;
 import org.apache.skywalking.banyandb.stream.v1.BanyandbStream;
 import org.apache.skywalking.banyandb.v1.client.grpc.exception.BanyanDBException;
+import org.apache.skywalking.banyandb.v1.client.metadata.MetadataCache;
 import org.apache.skywalking.banyandb.v1.client.metadata.Serializable;
 
-import java.util.Deque;
-import java.util.LinkedList;
-
 /**
  * StreamWrite represents a write operation, including necessary fields, for {@link
  * BanyanDBClient#buildStreamWriteProcessor}.
@@ -41,8 +42,11 @@ public class StreamWrite extends AbstractWrite<BanyandbStream.WriteRequest> {
     @Getter
     private final String elementId;
 
-    public StreamWrite(final String group, final String name, final String elementId, long timestamp) {
-        super(group, name, timestamp);
+    /**
+     * Create a StreamWrite without initial timestamp.
+     */
+    StreamWrite(MetadataCache.EntityMetadata entityMetadata, final String elementId) {
+        super(entityMetadata);
         this.elementId = elementId;
     }
 
@@ -51,13 +55,25 @@ public class StreamWrite extends AbstractWrite<BanyandbStream.WriteRequest> {
         return (StreamWrite) super.tag(tagName, tagValue);
     }
 
+    public void setTimestamp(long timestamp) {
+        super.timestamp = Optional.of(timestamp);
+    }
+
     /**
      * Build a write request
      *
      * @return {@link BanyandbStream.WriteRequest} for the bulk process.
      */
     @Override
-    protected BanyandbStream.WriteRequest build(BanyandbCommon.Metadata metadata, Timestamp ts) {
+    protected BanyandbStream.WriteRequest build(BanyandbCommon.Metadata metadata) {
+        if (!timestamp.isPresent() || timestamp.get() <= 0) {
+            throw new IllegalArgumentException("Timestamp is required and must be greater than 0 for stream writes.");
+        }
+
+        Timestamp ts = Timestamp.newBuilder()
+                .setSeconds(timestamp.get() / 1000)
+                .setNanos((int) (timestamp.get() % 1000 * 1_000_000)).build();
+
         final BanyandbStream.WriteRequest.Builder builder = BanyandbStream.WriteRequest.newBuilder();
         builder.setMetadata(metadata);
         final BanyandbStream.ElementValue.Builder elemValBuilder = BanyandbStream.ElementValue.newBuilder();
@@ -66,14 +82,13 @@ public class StreamWrite extends AbstractWrite<BanyandbStream.WriteRequest> {
         // memorize the last offset for the last tag family
         int lastFamilyOffset = 0;
         for (final int tagsPerFamily : this.entityMetadata.getTagFamilyCapacity()) {
-            final BanyandbModel.TagFamilyForWrite.Builder b = BanyandbModel.TagFamilyForWrite.newBuilder();
             boolean firstNonNullTagFound = false;
             Deque<BanyandbModel.TagValue> tags = new LinkedList<>();
             for (int j = tagsPerFamily - 1; j >= 0; j--) {
                 Object obj = this.tags[lastFamilyOffset + j];
                 if (obj == null) {
                     if (firstNonNullTagFound) {
-                        b.addTags(TagAndValue.nullTagValue().serialize());
+                        tags.addFirst(TagAndValue.nullTagValue().serialize());
                     }
                     continue;
                 }
@@ -81,9 +96,10 @@ public class StreamWrite extends AbstractWrite<BanyandbStream.WriteRequest> {
                 tags.addFirst(((Serializable<BanyandbModel.TagValue>) obj).serialize());
             }
             lastFamilyOffset += tagsPerFamily;
-            elemValBuilder.addTagFamilies(b.addAllTags(tags).build());
+            elemValBuilder.addTagFamilies(BanyandbModel.TagFamilyForWrite.newBuilder().addAllTags(tags).build());
         }
         builder.setElement(elemValBuilder);
+        builder.setMessageId(System.nanoTime());
         return builder.build();
     }
 }
