@@ -22,6 +22,7 @@ import java.util.List;
 
 import com.google.common.base.Strings;
 import com.google.protobuf.ByteString;
+import com.google.protobuf.Timestamp;
 import lombok.EqualsAndHashCode;
 import lombok.Getter;
 import org.apache.skywalking.banyandb.model.v1.BanyandbModel;
@@ -128,6 +129,31 @@ public abstract class Value<T> {
         }
     }
 
+    /**
+     * The value of a timestamp type tag.
+     */
+    public static class TimestampTagValue extends Value<Long> implements Serializable<BanyandbModel.TagValue> {
+        private TimestampTagValue(Long value) {
+            super(value);
+        }
+
+        @Override
+        public BanyandbModel.TagValue serialize() {
+            Timestamp timestamp = toProtobufTimestamp(value);
+            return BanyandbModel.TagValue.newBuilder().setTimestamp(timestamp).build();
+        }
+    }
+
+    /**
+     * Utility method to convert milliseconds to protobuf Timestamp.
+     */
+    private static Timestamp toProtobufTimestamp(long millis) {
+        return Timestamp.newBuilder()
+                .setSeconds(millis / 1000)
+                .setNanos((int) ((millis % 1000) * 1000000))
+                .build();
+    }
+
     /**
      * Construct a string tag
      *
@@ -193,6 +219,16 @@ public abstract class Value<T> {
         return new LongArrayTagValue(val);
     }
 
+    /**
+     * Construct a timestamp tag
+     *
+     * @param epochMilli payload in milliseconds
+     * @return Anonymous tag with timestamp payload
+     */
+    public static Serializable<BanyandbModel.TagValue> timestampTagValue(long epochMilli) {
+        return new TimestampTagValue(epochMilli);
+    }
+
     public static Serializable<BanyandbModel.TagValue> nullTagValue() {
         return NullTagValue.INSTANCE;
     }
@@ -253,6 +289,20 @@ public abstract class Value<T> {
         }
     }
 
+    /**
+     * The value of a float type field.
+     */
+    public static class FloatFieldValue extends Value<Float> implements Serializable<BanyandbModel.FieldValue> {
+        private FloatFieldValue(Float value) {
+            super(value);
+        }
+
+        @Override
+        public BanyandbModel.FieldValue serialize() {
+            return BanyandbModel.FieldValue.newBuilder().setFloat(BanyandbModel.Float.newBuilder().setValue(value)).build();
+        }
+    }
+
     /**
      * Construct a numeric tag
      *
