@@ -21,31 +21,32 @@ package org.apache.skywalking.banyandb.v1.client.metadata;
 import io.grpc.Channel;
 import org.apache.skywalking.banyandb.common.v1.BanyandbCommon;
 import org.apache.skywalking.banyandb.database.v1.BanyandbDatabase;
+import org.apache.skywalking.banyandb.database.v1.BanyandbDatabase.IndexRuleBinding;
 import org.apache.skywalking.banyandb.database.v1.IndexRuleBindingRegistryServiceGrpc;
 import org.apache.skywalking.banyandb.v1.client.grpc.MetadataClient;
 import org.apache.skywalking.banyandb.v1.client.grpc.exception.BanyanDBException;
 
 import java.util.List;
-import java.util.stream.Collectors;
 
 public class IndexRuleBindingMetadataRegistry extends MetadataClient<IndexRuleBindingRegistryServiceGrpc.IndexRuleBindingRegistryServiceBlockingStub,
-        BanyandbDatabase.IndexRuleBinding, IndexRuleBinding> {
+        BanyandbDatabase.IndexRuleBinding> {
 
     public IndexRuleBindingMetadataRegistry(Channel channel) {
         super(IndexRuleBindingRegistryServiceGrpc.newBlockingStub(channel));
     }
 
     @Override
-    public void create(IndexRuleBinding payload) throws BanyanDBException {
+    public long create(IndexRuleBinding payload) throws BanyanDBException {
         execute(() -> stub.create(BanyandbDatabase.IndexRuleBindingRegistryServiceCreateRequest.newBuilder()
-                .setIndexRuleBinding(payload.serialize())
+                .setIndexRuleBinding(payload)
                 .build()));
+        return DEFAULT_MOD_REVISION;
     }
 
     @Override
     public void update(IndexRuleBinding payload) throws BanyanDBException {
         execute(() -> stub.update(BanyandbDatabase.IndexRuleBindingRegistryServiceUpdateRequest.newBuilder()
-                .setIndexRuleBinding(payload.serialize())
+                .setIndexRuleBinding(payload)
                 .build()));
     }
 
@@ -65,7 +66,16 @@ public class IndexRuleBindingMetadataRegistry extends MetadataClient<IndexRuleBi
                         .setMetadata(BanyandbCommon.Metadata.newBuilder().setGroup(group).setName(name).build())
                         .build()));
 
-        return IndexRuleBinding.fromProtobuf(resp.getIndexRuleBinding());
+        return resp.getIndexRuleBinding();
+    }
+
+    @Override
+    public ResourceExist exist(String group, String name) throws BanyanDBException {
+        BanyandbDatabase.IndexRuleBindingRegistryServiceExistResponse resp = execute(() ->
+                stub.exist(BanyandbDatabase.IndexRuleBindingRegistryServiceExistRequest.newBuilder()
+                        .setMetadata(BanyandbCommon.Metadata.newBuilder().setGroup(group).setName(name).build())
+                        .build()));
+        return ResourceExist.create(resp.getHasGroup(), resp.getHasIndexRuleBinding());
     }
 
     @Override
@@ -75,6 +85,6 @@ public class IndexRuleBindingMetadataRegistry extends MetadataClient<IndexRuleBi
                         .setGroup(group)
                         .build()));
 
-        return resp.getIndexRuleBindingList().stream().map(IndexRuleBinding::fromProtobuf).collect(Collectors.toList());
+        return resp.getIndexRuleBindingList();
     }
 }
